#!/usr/bin/env perl

package Prty::Application::PodcastNdrinfoZeitzeichenDownload::Test;
use base qw/Prty::Test::Class/;

use strict;
use warnings;

# -----------------------------------------------------------------------------

sub test_loadClass : Init(1) {
    shift->useOk('Prty::Application::PodcastNdrinfoZeitzeichenDownload');
}

# -----------------------------------------------------------------------------

package main;
Prty::Application::PodcastNdrinfoZeitzeichenDownload::Test->runTests;

# eof
