package P1;

our $x1 = 1; # mit our vereinbarte skalare Variable müssen def. Wert haben
our $y1 = 1;
our @y1;
our @z1;
our %z1;

use vars '$x2';
use vars '$y2';
use vars '@y2';
use vars '@z2';
use vars '%z2';

1;
